/* RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and that
no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


*/

#include "rwhois.h"

/* Process the command line arguments */

procargs( argv, retline)
char **argv;
char *retline;
{
int clport, cltype;
char temp[MAXBUFF];
char *dirbuff;
int change_server = FALSE;
char *tmpdir;
char tmpline[80];
int multiple = 0;
struct SERVER tmp_server;
int ret = GOOD;

	tmpline[0] = NullCH;

	while( *argv)
	{
		upper( *argv);

		if (!strcmp(*argv, "-H") || !strcmp(*argv, "-A")) 
		{
			if(!strcmp( *argv, "-H"))
				SERVERREWIND;

			if(!change_server)
			{
				change_server = TRUE;
				current_referral = 0;
				referral = 0;
			}

			++argv;

			brk_server( *argv, &clport, &cltype);

			tmp_server.server = *argv;
			tmp_server.port = clport;
			tmp_server.type_status = cltype;
			tmp_server.reftype = REF_CMDLINE;
			tmp_server.referral = NULL;
			tmp_server.query = NULL;
			tmp_server.filename = NULL;
			tmp_server.filestatus = 0;

			server_main( SER_ADD, &tmp_server);

		}
		else if (!strcmp(*argv, "-S")) 
			shots = SINGLESHOT;
		else if (!strcmp(*argv, "-M"))
			shots = MULTISHOT;
		else if (!strcmp(*argv, "-V")) 
			verbose = TALK;
		else if (!strcmp(*argv, "-D")) 
			verbose = SHOUT;
		else if (!strcmp(*argv, "-Q")) 
			verbose = QUIET; 
		else if(!strcmp(*argv, "-K"))
		{
			goodedit = REG_DEL;
			edit = ON;
			to_file = TRUE;
			if( verbose > WHISPER )
			{
				note( "Object delete mode set");
			}
		}
		else if(!strcmp(*argv, "-E"))
		{
			goodedit = REG_MOD;
			edit =	ON; 
			to_file = TRUE;
			if( verbose > WHISPER )
			{
				note( "Object edit mode set");
			}
		}
		else if (!strcmp(*argv, "-C")) 
		{
			++argv;

			if( argv)
			{
				if( confdir)
				{
					dirbuff = safecalloc( 1,(strlen( confdir) + strlen( *argv)+2)); 
					sprintf(  dirbuff, "%s%s%s", confdir, 
						confdir[strlen(confdir)] == '/' ? "" : "/", *argv);
					if( procrc( dirbuff) == -1 )
						procrc( *argv);
					free( temp);
				}
				else
					procrc( *argv);

			}
			else
			{
				error("Configuration file name required following the -c option");
				ret = BAD;
			}
		}
		else if(!strcmp(*argv, "-X"))
		{
			++argv;

			if( argv)
			{
				precmd = safecalloc( 1, (strlen( *argv)+1));
				strcpy( precmd,  *argv);
				control_pannel[CP_PRECMD] = SET_SEND;
				if( verbose > WHISPER )
				{
					sprintf( temp, "Extended command set for [%s]", precmd);
					note( temp);
				}
			}
			else
			{
				error("Directive requried following the -x option");
				ret = BAD;
			}
		}
		else if(!strcmp(*argv, "-F"))
		{
			++argv;

			if( argv)
			{
				tmpdir = safecalloc( 1, (strlen( *argv)+1));
				strcpy( tmpdir,  *argv);
				setsend( tmpdir);
				if( verbose > WHISPER )
				{
					sprintf( temp, "Command set for [%s]", tmpdir);
					note( temp);
				}
				free( tmpdir);
			}
			else
			{
				error("Directive required follwoing the -f option");
				ret = BAD;
			}
		}
		else if(!strncmp(*argv, "-R", 2))
		{
			if( (int)strlen( *argv) < 3 )
			{
				printf("\nNeed the regtype ( -ra = add -rd = delete -rm = modify\n");
				exit(-1);
			}

			if(!strcmp( *argv, "-RA" ))
			{
				regtype = REG_ADD;
				if( verbose > WHISPER )
					note("Registration type set to add");
			}
			if(!strcmp( *argv, "-RM" ))
			{
				regtype = REG_MOD;
				if( verbose > WHISPER )
					note("Registration type set to modify");
			}
			if(!strcmp( *argv, "-RD" ))
			{
				regtype = REG_DEL;
				if( verbose > WHISPER )
					note("Registration type set to delete");
			}

			++argv;

			if( argv)
			{
				regfile = safecalloc( 1, (strlen( *argv)+1));
				strcpy( regfile,  *argv);
				reg_only = TRUE;
				if( verbose > WHISPER )
				{
					sprintf( temp, "Will send [%s] registration file", regfile);
					note( temp);
				}
			}
			else
			{
				error("Registration file must follwoing the -r option");
				ret = BAD;
			}
		}
		else if(!strcmp(*argv, "-T"))
		{
			authority = TRUE;
			control_pannel[CP_SOA] = SET_SEND|SET_ALWAYS;
		}
		else if(!strcmp(*argv, "-U"))
		{
			++argv;
			if( argv)
			{
				user = safecalloc( 1, (strlen( *argv)+1));
				strcpy( user,  *argv);
				if( verbose > WHISPER )
				{
					sprintf(temp, "User set to [%s]", user);
					note( temp);
				}
			}
			else
			{
				error("User id required following the -u option");
				ret = BAD;
			}

		}
		else if(!strcmp(*argv, "-P"))
		{
			++argv;
			if( argv)
			{
				authstring = safecalloc( 1, (strlen( *argv)+1));
				strcpy( authstring,  *argv);
				if( verbose > WHISPER )
					note("Authority string set");
			}
			else
			{
				error("Authoirity string required following the -p option");
				ret = BAD;
			}

		}
		else if(!strcmp(*argv, "--"))
			just_show_config = TRUE;
		else if(!strcmp(*argv, "-+"))
			preserve_temp = TRUE;
		else 
		{
		
			if( multiple )
			 	strcat( tmpline, " ");	
			strcat( tmpline, *argv);
			++multiple;
		}
		++argv;
	}

	strcpy( retline, tmpline);

	return( ret);
}
