#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "fileinfo.h"

/**********
  current globals
***********/

static mkdb_fileinfo_struct *fileinfo;

/**********
  local prototypes
**********/

static file_type select_type();

/****************************************************************************
  reads the files for the database
  returns TRUE if ok
          FALSE if not
****************************************************************************/
int read_fileinfo (filename)
char *filename;

{
FILE *fp;
mkdb_fileinfo_struct *top = NULL, *cur, *new;

/** this may be for the global type **/
if((fp = fopen(filename, "r")) == (FILE *) NULL){
  print_error (999, "read_fileinfo: can not open %s", filename);
  return(FALSE);
  }
fileinfo = load_fileinfo_list (fp);
return (TRUE);
}

/****************************************************************************
  loads the fileinfo structure from a open file pointer
  returns a pointer to the fileinfo struct
****************************************************************************/
mkdb_fileinfo_struct *load_fileinfo_list (fp)
FILE *fp;

{
char line[MAX_LINE];
char tag[MAX_LINE];
char datum[MAX_LINE];
mkdb_fileinfo_struct *top = NULL, *cur, *new;

while ((fgets (line, MAX_LINE, fp)) != NULL) {
  stripchar (line, '\n');
  if (new_record (line) || top == NULL) {
    new = NEW_FILE_INFO;
    if (top == NULL)
      top = cur = new;
    else {
      cur->nxt = new;
      cur = new;
      }
    }
  if (parse_line (line, tag, datum)) {
    if (strcmp (tag, MKDB_TYPE) == 0)
      cur->type = select_type (datum);
    else if (strcmp (tag, MKDB_FILE) == 0)
      cur->filename = NEW_STRING (datum);
    else if (strcmp (tag, MKDB_NUMRECS) == 0)
      cur->num_recs = atoi(datum);
    else if (strcmp (tag, MKDB_INDEX_NO) == 0)
      cur->index_no = atoi(datum);
    else if (strcmp (tag, MKDB_SIZE) == 0)
      cur->size = atoi (datum);
    }
  }
return (top);
}

/****************************************************************************
  returns how to index the word based on the attribute
  if none declared, returns all words.
****************************************************************************/
static file_type select_type (ftype)
char *ftype;

{
if (strcasecmp (ftype, MKDB_INDEX_FILE) == 0)
  return (ENUM_INDEX_FILE);
else 
  return (ENUM_DATA_FILE);
}

/****************************************************************************
  clears the file info from memory
****************************************************************************/
clear_fileinfo ()
{
clear_fileinfo_list (fileinfo);
}

/****************************************************************************
  clears the file info from memory
****************************************************************************/
clear_fileinfo_list (cur)
mkdb_fileinfo_struct *cur;

{
mkdb_fileinfo_struct  *del;

while (cur != NULL) {
  free (cur ->filename);
  del = cur;
  cur = cur->nxt;
  free (del);
  }
}


/****************************************************************************
  returns the info for the specified type
    NULL if does not exist
****************************************************************************/
mkdb_fileinfo_struct *get_fileinfo (type, index_no)
file_type type;
int index_no;

{
mkdb_fileinfo_struct *cur;

cur =  fileinfo;
while (cur != NULL) {
  if (type == ENUM_INDEX_FILE && cur->type == ENUM_INDEX_FILE && 
    cur->index_no >= index_no) {
    return (cur);
    }
  else if (cur->index_no == index_no && cur->type != ENUM_INDEX_FILE
     && type != ENUM_INDEX_FILE)
    return (cur);
  cur = cur->nxt;
  }
return (NULL); 
}

/****************************************************************************
  allows for a scan of all data files within a specific index
****************************************************************************/
char *get_fileinfo_data_files (reset)
int reset;

{
static mkdb_fileinfo_struct *cur = NULL;
char *ret = NULL;

if (reset)
  cur = fileinfo;

while (cur != NULL && ret == NULL) {
  if (cur->type == ENUM_DATA_FILE)
    ret = cur->filename;
  cur = cur->nxt;
  }
return (ret);
}

/****************************************************************************
  retrieves number of records
****************************************************************************/
int get_fileinfo_record_count ()

{
mkdb_fileinfo_struct *cur = fileinfo;
int num_recs=0;

while (cur != NULL) {
  num_recs += cur->num_recs;
  cur = cur->nxt;
  }
return (num_recs);
}

/****************************************************************************
  returns the debugged info
****************************************************************************/
debug_fileinfo()
{
mkdb_fileinfo_struct *cur;

cur =  fileinfo;
while (cur != NULL) {
  if (cur->type == ENUM_INDEX_FILE)
    fprintf (stderr, "type: |%s|\n", MKDB_INDEX_FILE);
  else
    fprintf (stderr, "type: |%s|\n", MKDB_DATA_FILE);
  fprintf (stderr, "index_no: |%d|\n", cur ->index_no);
  fprintf (stderr, "filename: |%s|\n", cur ->filename);
  fprintf (stderr, "size: |%d|\n", cur ->size);
  fprintf (stderr, "numrecs: |%d|\n", cur ->num_recs);
  cur = cur->nxt;
  }
}
