/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include "conf.h"
#include "llist.h"

#ifndef _ATTRIBUTES_H_
#define _ATTRIBUTES_H_

/****
  tags for parsing definitions
****/

#define A_ATTRIBUTE      "attribute"
#define A_ATTRIB_ALIAS   "attribute-alias"
#define A_DESCRIPTION    "description"
#define A_FORMAT         "format"
#define A_IS_PRIMARY_KEY "is-primary-key"
#define A_IS_AUTH_AREA   "is-authority-area"
#define A_IS_SOA_SERIAL  "is-soa-serial"
#define A_IS_REQUIRED    "required"
#define A_MULT_LINES     "mult-lines"
#define A_INDEX          "index"
#define A_DEF_OBJECT     "def_object"
#define A_MULTIPLE_GRP   "is-multiple-grp"
#define A_TYPE           "type"

/***
  for rwaisindex  indexing
***/

#define A_INDEX_ALL_WORDS   "ALL"
#define A_INDEX_ONLY_WORDS  "INDEX_ONLY"
#define A_INDEX_FIRST_WORDS "FIRST"
#define A_INDEX_NONE_WORDS  "NONE"

typedef enum {
  INDEX_ALL_WORDS,
  INDEX_FIRST_WORD,
  INDEX_WORD_ONLY,
  INDEX_NO_WORDS
} a_index_type;

/***
  attribute types
***/

#define A_TYPE_TEXT      "TEXT"
#define A_TYPE_FILE      "FILE"
#define A_TYPE_MIME      "MIME"
#define A_SEE_ALSO       "SEE-ALSO"
#define A_REFERRAL       "REFERRAL"

typedef enum {
  TYPE_TEXT,
  TYPE_FILE,
  TYPE_MIME,
  TYPE_SEE_ALSO,
  TYPE_REFERRAL,
} a_type;

#define  INDEX_PRFX       "QWQX"

/* define the attributes within an object */
typedef struct _attribute_struct {
  int   global_id;                 /* unique id for all objects */
  int   internal_id;               /* unique id within the object */
  int   primary_key;               /* TRUE if primary key attribute */
  int   authority_area;            /* TRUE if a authority area attribute */
  int   soa;                       /* TRUE if a soa serial# attribute */
  int   required;                  /* TRUE if a required attribute */
  int   mult_lines;                /* TRUE if a can have mult lines */
  int   def_object;                /* TRUE if defines object */
  int   multiple_grp;              /* TRUE if defines multiple grouping */
  char *name;                      /* name of the attribute */
  char *name_alias;                /* alias name for the attribute */
  char *description;               /* description of the attribute */
  char *format;                    /* format of the object */
  char *object_name;               /* name of the object */
  a_index_type index_words;        /* what to index for rwaisindex */
  a_type type;                     /* type of attribute */
  int no_object_references;        /* number of linked list references */
  llist_struct *object_references; /* linked list of object references */
  struct _attribute_struct *nxt;   /* next item in list for that object */
} attribute_struct;

#define NEW_ATTRIBUTE (attribute_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(attribute_struct))

/***********
  prototypes
***********/
extern char *get_attribute_name();
extern attribute_struct *get_attribute_info_from_global_id();
extern attribute_struct *get_attribute_info_from_internal_id();
extern attribute_struct *set_attributes_list_from_object();
extern attribute_struct *get_attribute_info_from_name_and_object();
extern attribute_struct *find_specific_attribute();
extern llist_struct *global_list_of_attributes();
extern char *show_attribute_type();
#endif
