/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "parse.h"
#include "commands.h"
#include "directive.h"

/** these are included for function names **/
#include "load.h"
#include "limit.h"
#include "schema.h"
#include "client_info.h"
#include "xfer.h"
#include "notify.h"
#include "holdconnect.h"
#include "exit.h"
#include "help.h"
#include "soa.h"
#include "cache.h"
#include "status.h"
#include "forward.h"
#include "register.h"
#include "debug.h"
#include "object.h"
#include "commands.h"
#include "define.h"
#include "display.h"
#include "private.h"
#include "pgp.h"

/****************************************************************************
 initializes the commands
****************************************************************************/

initialize_directive_list()
{
append_command ("define", 6, NULL, "define", FALSE, COMMAND_DIRECTIVE, NULL,
  display_define, NULL);
append_command ("directive", 9, NULL, "directive", FALSE, COMMAND_DIRECTIVE, 
  NULL, display_directive, NULL);
append_command ("object", 6, NULL, "object", FALSE, COMMAND_DIRECTIVE, NULL,
  display_object, NULL);
append_command ("X-debug", 7, NULL, "debug mode", FALSE, COMMAND_DIRECTIVE, NULL,
  set_debug, NULL);
append_command ("X-pgpid", 7, NULL, "debug mode", FALSE, COMMAND_DIRECTIVE, NULL,
  xpgpid, NULL);
append_command ("register", 8, NULL, "register object", FALSE, 
  COMMAND_DIRECTIVE, NULL, record_register, NULL);
append_command ("private", 7, NULL, "private", FALSE, COMMAND_DIRECTIVE, NULL,
  set_private, NULL);
append_command ("notify", 6, NULL, "notify", FALSE, COMMAND_DIRECTIVE, NULL,
  record_notify, NULL);
append_command ("forward", 7, NULL, "forward", FALSE, COMMAND_DIRECTIVE, NULL,
  set_forward, NULL);
append_command ("status", 6, NULL, "status", FALSE, COMMAND_DIRECTIVE, NULL,
  show_status, NULL);
append_command ("cache", 5, NULL, "cache boolean", FALSE, COMMAND_DIRECTIVE, 
  NULL, set_cache, NULL);
append_command ("soa", 3, NULL, "displays authoritative area for server", 
  FALSE, COMMAND_DIRECTIVE, NULL, show_soa, NULL);
append_command ("load", 4, NULL, "displays system load average", FALSE, 
  COMMAND_DIRECTIVE, NULL, display_system_load, NULL);
append_command ("schema", 6, NULL, "displays schema objects and attributes", 
  FALSE, COMMAND_DIRECTIVE, NULL, display_schema, NULL);
append_command ("limit", 5, NULL, "displays and sets record hit limit", 
  FALSE, COMMAND_DIRECTIVE, NULL, set_max_hit_limit, NULL);
append_command ("rwhois", 6, NULL, "identifies rwhois client", FALSE, 
  COMMAND_DIRECTIVE, NULL, set_client_info, NULL);
append_command ("holdconnect", 11, NULL, "hold the connection boolean", FALSE, 
  COMMAND_DIRECTIVE, NULL, set_holdconnect, NULL);
append_command ("display", 7, NULL, "sets/displays the display types", FALSE, 
  COMMAND_DIRECTIVE, NULL, set_or_display_display, NULL);
append_command ("xfer", 4, NULL, "transfer of objects/authority areas", 
  FALSE, COMMAND_DIRECTIVE, NULL, transfer_data, NULL);
/*
append_command ("help", 4, NULL, "display help text", FALSE, 
  COMMAND_DIRECTIVE, NULL, help, NULL);
*/
append_command ("quit", 4, NULL, "exit rwhois", FALSE, COMMAND_DIRECTIVE, NULL,
  exit_whois, NULL);
/*
append_command ("exit", 4, NULL, "exit rwhois", FALSE, COMMAND_DIRECTIVE, NULL,
  exit_whois, NULL);
*/
}

/****************************************************************************
 reads the extended directives from a file
****************************************************************************/
read_extended_directive_file (file)
char *file;

{
FILE *fp;
char line[BUFSIZ];
char tag[MAX_TEMPLATE_DESC];
char datum[MAX_TEMPLATE_DESC];
char description[MAX_TEMPLATE_DESC];
char command[MAX_TEMPLATE_DESC];
int command_len;
char program[MAX_TEMPLATE_DESC];

if (strlen(file) == 0)
  return (TRUE);

*command = '\0';
*program = '\0';
*description = '\0';
command_len = 0;

if((fp = fopen(file, "r")) == (FILE *) NULL){
  return(FALSE);
  }

while ((fgets (line, BUFSIZ, fp)) != NULL) {
  if (new_record (line) && strlen (command) && strlen (program)) {
    if (command_len == 0)
      command_len = strlen (command);
    append_command (command, command_len, NULL, description, FALSE, 
       COMMAND_X_DIRECTIVE, NULL, NULL, program);
    *command = '\0';
    *program = '\0';
    command_len = 0;
    }
  else if (parse_line (line, tag, datum)) {
    if (strcasecmp (tag, D_COMMAND) == 0)
      sprintf (command, "X-%s", datum);
    else if (strcasecmp (tag, D_COMMAND_LEN) == 0)
      command_len = atoi (datum) + 2;
    else if (strcasecmp (tag, D_COMMAND_DESCRIPTION) == 0)
      strcpy (description, datum);
    else if (strcasecmp (tag, D_COMMAND_PROGRAM) == 0)
      strcpy (program, datum);
    }
  }
if (strlen (command) && strlen (program)) {
  if (command_len == 0)
    command_len = strlen (command);
  append_command (command, command_len, NULL, description, FALSE, 
    COMMAND_X_DIRECTIVE, NULL, NULL, program);
  }
return (TRUE);
}

/*************************************************************************
  returns TRUE if is directive command
          FALSE otherwise
**************************************************************************/
is_directive_from_str (str)
char *str;

{
if (str[0] == '-')
  return (TRUE);
return (FALSE);
}

/*************************************************************************
  returns TRUE if is directive command
          FALSE otherwise
**************************************************************************/
is_directive (query_item)
query_item_struct *query_item;

{
if (query_item->directive != NULL)
  return (TRUE);
return (FALSE);
}

/*************************************************************************
  creates the directive into a query structure
**************************************************************************/
query_item_struct *create_directive (str)
char *str;

{
int i, done = FALSE;
query_item_struct *new;
char word[MAX_LINE], *cp, *next;

cp = str+1; /* skips the "-" symbol */
next = get_word (cp, word);
new = NEW_QUERY_ITEM;
new->directive = NEW_STRING (word);
new->value = NEW_STRING (next);
do_commands(word, next, new, TRUE);
return (new);
}

/*************************************************************************
  runs the directive
  returns TRUE on success
          FALSE on failure
**************************************************************************/
int run_directive (query_item)
query_item_struct *query_item;

{
if (query_item->function == NULL && query_item->program == NULL) {
  print_error (INVALID_DIRECTIVE, "");
  set_passthru();
  }
#if AUTHORIZATION_ON
else if (!authorized_directive (query_item->directive)) {
  if (query_item->function)
    print_error (UNAUTH_DIRECTIVE, "");
  else
    print_error (UNAUTH_X_DIRECTIVE, "");
  log_entry (RWHOIS_LOG_FILE, "Ans: Directive DENIED");
  set_passthru();
  }
#endif
else if (!is_client_capable_of_action (query_item->directive)) {
  print_error (UNCOMPAT_VERSION, "");
  set_passthru();
  }
else if (query_item->function)
  (query_item->function)(query_item->value);
else if (run_program (query_item->program, query_item->value)) {
  print_ok();
  set_passthru();
  }
}
