/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "displays.h"
#include "display.h"

/****************************************************************************
  calls set or display based on passed value
****************************************************************************/
set_or_display_display (str)
char *str;

{
char word[MAX_LINE], *cp, *next, *fname;
int retval;

cp = str;
next = get_word (cp, word);

if (strlen (str) == 0)
  retval = display_display(return_all_defined_displays());
else if (strlen (next) == 0) 
  retval = set_display(str, return_all_defined_displays());
else if (strcasecmp (word, "activate") == 0)
  retval = set_display(next, return_all_defined_displays());
else if (strcasecmp (word, "capable") == 0)
  retval = set_client_capabilities (next);
else {
  print_error (INVALID_DIRECTIVE_PARAM, "");
  set_passthru();
  return (FALSE);
  }
if (!retval)
  print_error (UNAUTH_DISPLAY, "");
else
  print_ok();
set_passthru();
}

/****************************************************************************
  sets the display type
****************************************************************************/
set_display(str, cur)
char *str;
output_command_struct *cur;

{
char *cp, *next, word[MAX_LINE];
int retval = FALSE;

cp = str;
next = get_word (cp, word);
if (is_valid_display_command(word, cur)) {
  if (strlen (next))
    retval = set_display_status (next, word);
  else 
    retval = set_display_status ("all", word);
  }
return (retval);
}

/****************************************************************************
 returns TRUE if valid display command
         FALSE if not
****************************************************************************/
is_valid_display_command(str, cur)
char *str;
output_command_struct *cur;

{
int length = strlen (str);

while (cur != NULL) {
  if (cur->command != NULL && length >= cur->command_len && 
    strncasecmp (str, cur->command, cur->command_len) == 0)
    return (TRUE);
  cur=cur->nxt;
  }
return (FALSE);
}

/****************************************************************************
  shows the display type  
****************************************************************************/
display_display(cur)
output_command_struct *cur;

{
int cnt = 0;

while (cur != NULL) {
  if (cur->command != NULL) {
    if (cnt++)
      printf ("%%display\n");
    printf ("%%display name: %s\n", cur->command);
    printf ("%%display length: %d\n", cur->command_len);
    printf ("%%display description: %s\n", cur->description);
    }
  cur = cur->nxt;
  }
}
