/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include <stdio.h>
#include "defines.h"
#include "client_msgs.h"
#include "objects.h"
#include "read_config.h"

/**********
  hidden defines
**********/

#define DAEMON_SERVER 0
#define INETD_SERVER 1

/***********
  local global things
***********/
static char hosts_allow[MAX_LINE];
static char hosts_deny[MAX_LINE];
static char hostname[MAX_LINE];
static char port[MAX_LINE];
static char updates_dir[MAX_LINE];
static char process_userid[MAX_LINE];
static char bin_path[MAX_LINE];
static char update_prog[MAX_LINE];
static int servertype;

/************
  hidden procs
*************/
static void set_local_port_from_config();
static void set_local_host_from_config();
static void set_updates_dir_from_config();
static void set_server_type_from_config();
static void set_process_userid();
static void set_binary_path();
static void set_update_prog();

/*************************************************************************
  initialize enviroment - sets up linked lists
  returns TRUE if okay
          FALSE otherwise
**************************************************************************/
read_config_files (config_file, in_jail)
char *config_file;
int *in_jail;

{
char display_file[MAX_LINE];
char directive_file[MAX_LINE];
char define_file[MAX_LINE];
char object_file[MAX_LINE];

if (!read_config_file (config_file, in_jail, display_file, directive_file,
  define_file, object_file) ) {
  print_error (MISC_DIAG, "can not open config file '%s'\n", config_file);
  return (FALSE);
  }
if (!read_extended_directive_file (directive_file)) {
  print_error (MISC_DIAG, "can not open extended directive file '%s'\n", 
    directive_file);
  return (FALSE);
  }
if (!read_define_file (define_file)) {
  print_error (MISC_DIAG, "can not open define file '%s'\n", define_file);
  return (FALSE);
  }
if (!read_displays(display_file)) {
  print_error (MISC_DIAG, "can not open displays file '%s'\n", display_file);
  return (FALSE);
  }
if (!read_objects(object_file)) {
  print_error (MISC_DIAG, "can not open object file '%s'\n", object_file);
  return (FALSE);
  }
return (TRUE);
}

/*************************************************************************
  reads the configuration file returns pointers to the display
  and object files.
  NOTE: any changes to the object-file stuff should also be placed
  into read_config_file_for_objects_only
  returns TRUE if ok
          FALSE otherwise
**************************************************************************/
read_config_file (config_file, in_jail, display_file, directive_file,
  define_file, object_file)
char *config_file;
int in_jail;
char *display_file;
char *directive_file;
char *define_file;
char *object_file;

{
FILE *fp;
char line[BUFSIZ];
char tag[MAX_TEMPLATE_DESC];
char datum[MAX_TEMPLATE_DESC];

*object_file = '\0';
*display_file = '\0';
*directive_file = '\0';
*define_file = '\0';
*hosts_allow = '\0';
*hosts_deny = '\0';
*hostname = '\0';
*port = '\0';

if((fp = fopen(config_file, "r")) == (FILE *) NULL)
  return(FALSE);
while ((fgets (line, BUFSIZ, fp)) != NULL) {
  if (parse_line (line, tag, datum)) {
    if (strcasecmp (tag, I_OBJECT_FILE) == 0)
      strcpy (object_file, datum);
    else if (strcasecmp (tag, I_DISPLAY_FILE) == 0)
      strcpy (display_file, datum);
    else if (strcasecmp (tag, I_DIRECTIVE_FILE) == 0)
      strcpy (directive_file, datum);
    else if (strcasecmp (tag, I_DEFINE_FILE) == 0)
      strcpy (define_file, datum);
    else if (strcasecmp (tag, I_MAX_HITS_DEFAULT) == 0)
      set_max_hits_default (atoi(datum));
    else if (strcasecmp (tag, I_MAX_HITS_CEILING) == 0)
      set_max_hits_ceiling (atoi(datum));
    else if (strcasecmp (tag, I_NOTIFY_LOG) == 0)
      set_notify_log(datum);
    else if (strcasecmp (tag, I_HOSTNAME) == 0)
      set_local_host_from_config(datum);
    else if (strcasecmp (tag, I_PORT) == 0)
      set_local_port_from_config(datum);
    else if (strcasecmp (tag, I_DEADMAN) == 0)
      set_deadman_time(datum);
    else if (strcasecmp (tag, I_DEFAULT_DIR) == 0) {
      if (!in_jail) 
        set_root_dir(datum);
      }
    else if (strcasecmp (tag, I_REGISTER_LOG) == 0)
      set_register_log(datum);
    else if (strcasecmp (tag, I_REGISTER_SPOOL) == 0)
      set_register_spool(datum);
    else if (strcasecmp (tag, I_UPDATES_DIR) == 0)
      set_updates_dir_from_config(datum);
    else if (strcasecmp (tag, I_SERVER_TYPE) == 0)
      set_server_type_from_config(datum);
    else if (strcasecmp (tag, I_PROC_USERID) == 0)
      set_process_userid(datum);
    else if (strcasecmp (tag, I_BIN_LOC) == 0)
      set_binary_path(datum);
    else if (strcasecmp (tag, I_UPDATE_PROG) == 0)
      set_update_prog(datum);
    else if (strcasecmp (tag, I_SECURITY_ALLOW) == 0)
      strcpy (hosts_allow, datum);
    else if (strcasecmp (tag, I_SECURITY_DENY) == 0)
      strcpy (hosts_deny, datum);
    else {
      printf ("config file object %s not found", tag);
      print_error (UNIDENT_ERROR, "config file object %s not found", tag); 
      }
    }
  }
fclose (fp);
return (TRUE);
}

/*************************************************************************
  returns pointers to the security files
**************************************************************************/
get_security_files (allow, deny)
char **allow, **deny;

{
*allow = hosts_allow;
*deny = hosts_deny;
}

/*************************************************************************
  retrieves the hostname
**************************************************************************/
char *get_local_host_from_config()
{
return (hostname);
}

/*************************************************************************
  sets the hostname
**************************************************************************/
static void set_local_host_from_config (host)
char *host;

{
strcpy (hostname, host);
}

/*************************************************************************
  retrieves the port
**************************************************************************/
char *get_local_port_from_config()
{
return (port);
}

/*************************************************************************
  sets the portname
**************************************************************************/
static void set_local_port_from_config(portno)
char *portno;

{
strcpy (port, portno);
}

/*************************************************************************
  gets the update dir for database updates
**************************************************************************/
char *get_updates_dir_from_config()
{
return (updates_dir);
}

/*************************************************************************
  sets the portname
**************************************************************************/
static void set_updates_dir_from_config(dir)
char *dir;

{
strcpy (updates_dir, dir);
}

/*************************************************************************
 returns true if daemon server, false if not 
**************************************************************************/
int is_daemon_server()
{
if (servertype == DAEMON_SERVER)
  return (TRUE);
return (FALSE);
}

/*************************************************************************
  sets the servertype
**************************************************************************/
static void set_server_type_from_config(servertypetext)
char *servertypetext;

{
if (strcasecmp (servertypetext, "INETD") == 0)
  servertype = INETD_SERVER;
else
  servertype = DAEMON_SERVER;
}

/*************************************************************************
  sets the the process user id
**************************************************************************/
static void set_process_userid(userid)
char *userid;

{
strcpy (process_userid, userid);
}


/*************************************************************************
  retrieves the the process user id
**************************************************************************/
char *get_process_userid()

{
return (process_userid);
}

/*************************************************************************
  sets the the binary path
**************************************************************************/
static void set_binary_path(path)
char *path;

{
strcpy (bin_path, path);
}


/*************************************************************************
  retrieves the the binary path
**************************************************************************/
char *get_binary_path()

{
return (bin_path);
}

/*************************************************************************
  sets the the update prog
**************************************************************************/
static void set_update_prog(prog)
char *prog;

{
strcpy (update_prog, prog);
}


/*************************************************************************
  retrieves the the update prog
**************************************************************************/
char *get_update_prog()

{
return (update_prog);
}

