/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "objects.h"
#include "log.h"
#include "badref.h"
#include "read_config.h"
#include "register.h"

/*******
  local prototypes and such
*******/

static char register_log[MAX_LINE];
static char register_spool[MAX_LINE];
static char register_program[MAX_LINE];
static void print_registration_header();
static char *setup_update_program();
static int valid_registration_header();
static int valid_registration_on_mode();
static int valid_registration_off_mode();
static int valid_registration_action();
static int valid_registration_email_address();
static int register_off();

char *get_config_file();

#define SPOOL_FILE_TEMPLATE "%s/%s.XXXXXX"
#define DB_FILE_TEMPLATE "%s/%s.XXXXXX"

/**************************************************************************
  records the registration stuff
**************************************************************************/
int record_register(str)
char *str;

{
char email_address[MAX_LINE];
char action[MAX_LINE];
char authority_info[MAX_LINE];
char line[MAX_LINE], fname[MAX_LINE], foname[MAX_LINE], params[MAX_LINE];
char nname[MAX_LINE];
int loop = TRUE, multiple_registration = FALSE;
llist_struct *object_list;
object_struct *object;
FILE *fp;
char *c;
int blanks = 0, run_bombed = FALSE;

if (!valid_registration_header(str, action, email_address, authority_info))
  print_error (INVALID_DIRECTIVE_PARAM, "");
else if (log_entry (register_log, "register: %s", str)) {
  if (create_filename (fname, SPOOL_FILE_TEMPLATE, register_spool) == NULL ||
    (fp = fopen (fname, "w")) == NULL) 
    print_error (UNIDENT_ERROR);
  else {
    print_ok();
    log_entry (register_log, "file: %s", fname);
    print_registration_header (fp, action, email_address);
    do {
      readline (line, MAX_LINE);
      stripchar (line, 13);
      if (register_off (line))
        loop = FALSE;
      else
        fprintf (fp, "%s\n", line);
      if (strlen(line) == 0) {
        multiple_registration = TRUE;
        blanks++;
        }
      else
        blanks = 0;
      }
    while (loop && blanks < 10);
    fclose (fp);
    object_list = return_all_defined_objects ("");
    while (object_list != NULL) {
      object = object_list->datum;
      if (object_in_registration (fname, object->name)) {
        if ((create_filename (foname, DB_FILE_TEMPLATE, object->dbdir)
          == NULL) || strcat (foname, ".txt") == NULL || 
          (fp = fopen (foname, "w")) == NULL) {
          print_error (UNIDENT_ERROR, "record_register open new registration \
file");
          run_bombed=TRUE;
          }
        else {
          save_like_objects (fname, object->name, action, email_address, fp);
          fclose (fp);
          if ((c = get_config_file(TRUE))!= NULL)
            sprintf (params, "-j -c %s %s", c, foname);
          else
            strcpy (params, foname);
          if (!run_program (setup_update_program(), params))
            run_bombed = TRUE;
          }
        }
      object_list=object_list->nxt;
      }
    }
  }
if (!run_bombed)
  print_ok();
set_passthru();
return(0);
}

/**************************************************************************
  records the header
**************************************************************************/
static int valid_registration_header(str, action, email_address, 
  authority_info)
char *str;
char *action;
char *email_address;
char *authority_info;

{
char *cp = str, *next, word[MAX_LINE];

next = get_word(cp, word);
if (!valid_registration_on_mode(word))
  return (FALSE);
cp = next;
next = get_word (cp, action);
if (!valid_registration_action(action))
  return (FALSE);
cp = next;
next = get_word (cp, email_address);
if (!valid_registration_email_address (email_address)) 
  return (FALSE);
if (!strlen (next))
  return (FALSE);
else
  strcpy (authority_info, next);
return (TRUE);
}

/**************************************************************************
  records the header
**************************************************************************/
static int valid_registration_on_mode(mode)
char *mode;

{
if (strcasecmp (mode, "on") == 0)
  return (TRUE);
return (FALSE);
}

/**************************************************************************
  records the header
**************************************************************************/
static int valid_registration_off_mode(mode)
char *mode;

{
if (strcasecmp (mode, "off") == 0)
  return (TRUE);
return (FALSE);
}

/**************************************************************************
  records the header
**************************************************************************/
static int valid_registration_action(action)
char *action;

{
if (strcasecmp (action, "add") == 0 || strcasecmp (action, "mod") == 0 || 
   strcasecmp (action, "del") == 0)
  return (TRUE);
return (FALSE);
}

/**************************************************************************
  records the header
**************************************************************************/
static int valid_registration_email_address(email)
char *email;

{
/****
  put vrfy code here!
***/

if (strlen (email))
  return (TRUE);
return (FALSE);
}

/**************************************************************************
  sets the register log directory
**************************************************************************/
int set_register_log (str)
char *str;

{
strcpy (register_log, str);
}

/**************************************************************************
  sets the register spool directory
**************************************************************************/
int set_register_spool (str)
char *str;

{
strcpy (register_spool, str);
}

/**************************************************************************
  sees if should turn off registration mode
**************************************************************************/
static int register_off (line)
char *line;

{
char *next, word[MAX_LINE];

if (*line != '-')
  return (FALSE);
next = get_word(++line, word);
if (strcasecmp (word, "register") == 0 || strcasecmp (next, "off") == 0)
  return (TRUE);
return (FALSE);
}

/**************************************************************************
  sets the registration header
**************************************************************************/
static void print_registration_header (fp, action, email_address)
FILE *fp;
char *action;
char *email_address;

{
fprintf (fp, "version: %s\n", REGISTRATION_VERSION);
fprintf (fp, "email: %s\n", email_address);
fprintf (fp, "action: %s\n", action);
}

/**************************************************************************
  setup the update prog
**************************************************************************/
static char *setup_update_program()
{

if (strlen (get_binary_path()))
  sprintf (register_program, "%s/%s", get_binary_path(), get_update_prog());
else
  strcpy (register_program, get_update_prog());
return (register_program);
}

/**************************************************************************
  is the object in the registration code?
**************************************************************************/
object_in_registration (fname, object_name)
char *fname;
char *object_name;

{
FILE *fp;
char line[MAX_LINE], object[MAX_LINE], attribute[MAX_LINE], datum[MAX_LINE];

if ((fp = fopen (fname, "r")) == NULL) {
  fprintf (stderr, "object_in_registration: can not open %s\n", fname);
  return (FALSE);
  }
while (fgets (line, MAX_LINE, fp) != NULL) {
  stripchar (line, '\n');
  trim (line);
  if (strlen (line) != 0) 
    get_tuple(object, attribute, datum, line);
  if (strcasecmp (object, object_name) == 0) {
    fclose (fp);
    return (TRUE);
    }
  }
fclose (fp);
return (FALSE);
}

/**************************************************************************
  save all the same objects in fp
**************************************************************************/
save_like_objects (fname, object_name, action, email_address, ofp)
char *fname;
char *object_name;
char *action;
char *email_address;
FILE *ofp;

{
FILE *ifp;
char line[MAX_LINE], object[MAX_LINE], attribute[MAX_LINE], datum[MAX_LINE];
int got_object = FALSE, multiple_regs = FALSE;

if ((ifp = fopen (fname, "r")) == NULL) {
  fprintf (stderr, "save_like_objects: can not open %s\n", fname);
  return (FALSE);
  }
print_registration_header (ofp, action, email_address);
while (fgets (line, MAX_LINE, ifp) != NULL) {
  stripchar (line, '\n');
  trim (line);
  if (strlen (line) == 0) {
    if (got_object) {
      multiple_regs = TRUE;
      got_object = FALSE;
      }
    }
  else {
    get_tuple(object, attribute, datum, line);
    if (strcasecmp (object, object_name) == 0) {
      /* put a separator in for rwhoisparse */
      if (multiple_regs && !got_object)
        fprintf (ofp, "\n");
      fprintf (ofp, "%s\n", line);
      got_object = TRUE;
      }
    }
  }
fclose (ifp);
}
