/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/

/**************************************************************************
  Change history

9/13/94 Included additional col in build_unsorted_index to mention that 
        element deleted (TRUE) or not (FALSE).
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"
#include "fileinfo.h"
#include "conf.h"
#include "attributes.h"
#include "client_msgs.h"
#include "io.h"
#include "rwhois_supp.h"

#define USAGE_STR "%s: <-c rwhois_conf_file>  <-m<i|a>> [-t<c|a>] <-q> <-j> <-e> [files]"

#define DATA_FILE             "DATA"
#define INDEX_FILE            "INDEX"

/************
  local prototypes
*************/

/*************************************************************************
  main routine
**************************************************************************/
main(argc, argv)
int argc;
char *argv[];

{
extern int optind;
extern char *optarg;
char c;
int num_recs=0;
int err=0;
int quiet=FALSE, in_jail = FALSE, ensure_in_jail = FALSE;
int mode = 0;
int type = AUTHORITATIVE;
FILE *dfp, *ifp;
char *config_file = NULL;
char *default_dir = NULL;
char relative_path_name[MAX_FILENAME];

/******
  initialization
******/
save_data_dir();

/******
  command line parsing
*******/

setbuf (stdout, NULL);
while ((c = getopt(argc, argv, "c:m:t:qje")) != EOF) {
  switch (c) {
    case 'c': /* config file */
      config_file = optarg;
      break;
    case 'm': /* mode options i for initialize a for append */
      if (strncasecmp (optarg, "i", 1) == 0)
        mode = INITIALIZE;
      else if (strncasecmp (optarg, "a", 1) == 0)
        mode = APPEND;
      else {
        print_error (MISC_DIAG, "invalid mode <a|i>!");
        err = 1;
        }
      break;
    case 't': /* type options c for cached a for authoritive */
      if (strncasecmp (optarg, "c", 1) == 0)
        type = CACHED;
      else if (strncasecmp (optarg, "a", 1) == 0)
        type = AUTHORITATIVE;
      else {
        print_error (MISC_DIAG, "invalid type <a|c>!");
        err = 1;
        }
      break;
    case 'j': /* in jail so in chrooted enviroment */
      in_jail = TRUE;
      break;
    case 'e': /* make sure data is in jail for a chrooted enviroment */
      ensure_in_jail = TRUE;
      break;
    case 'q': /* quiet (don't spew out messages) */
      quiet = TRUE;
      break;
    default:
      print_error (MISC_DIAG, USAGE_STR, argv[0]);
      err=1;
      break;
    }
  }

/*******
  required ops missing
********/

if (mode == 0 && !err) {
  print_error (MISC_DIAG, "missing mode!");
  err = 1;
  }
if (argc == 1 || err) {
  print_error (MISC_DIAG, "Exiting");
  exit (1);
  }

/********
  read in rwhois configuration items
********/

if (config_file == NULL)
  config_file = DEFAULT_RWHOIS_CONFIG_FILE;
if (!read_config_files(config_file, in_jail))  {
  print_error (MISC_DIAG, "error opening configuration files!");
  print_error (MISC_DIAG, "Exiting");
  printf ("MAK - bye\n");
  exit (1);
  }
save_rwhois_root();

/****** 
  show status
******/

if (!quiet)
  show_index_startup_status(mode, type, default_dir, config_file);

/*******
  change back to current directory and index
*******/

chdir_data_dir();
while (optind<argc) {
  if (ensure_in_jail) {
    sprintf (relative_path_name, "%s",
      chg_file_relative_to_rwhois_root(argv[optind]));
    if (strlen (relative_path_name) == 0) {
      print_error (MISC_DIAG, "%s is out of the rwhois root area!",
        argv[optind]);
      break;
      }
    }
  else {
    if (strlen (relative_to_rwhois_root()) > 0 && *argv[optind] != '/')
      sprintf (relative_path_name, "%s/%s", relative_to_rwhois_root(),
        argv[optind]); 
    else
      strcpy (relative_path_name, argv[optind]);
    }
  if (!quiet)
    printf ("indexing '%s'\n", relative_path_name);
  if (ensure_in_jail)
    chdir_rwhois_root();
  build_unsorted_index (mode, type, quiet, relative_path_name);
  if (ensure_in_jail)
    chdir_data_dir();
  optind++;
  } 
flush_and_sort (quiet, in_jail);
}

/*************************************************************************
 builds the index file - places in format
   offset:fileno:deleted:data
  RETURNS number of records
**************************************************************************/
int build_unsorted_index (mode, type, quiet, data_file)
int mode;
int type;
int quiet;
char *data_file;

{
char attribute_name[MAX_LINE];
char object_name[MAX_LINE];
char datum[MAX_LINE];
char line[MAX_LINE];
long offset=0;
int line_no = 0;
int file_no;
FILE *fp, *ifp;
int pass;
int do_increment_rec_count;

if ((fp = open_file (data_file, "r")) == NULL) {
  print_error (MISC_DIAG, "build_unsorted_index: can not open data file '%s'",
    data_file);
  return (FALSE);
  }

do_increment_rec_count = TRUE;
*object_name = '\0';
while (fgets (line, MAX_LINE, fp) != NULL) {
  if (!(++line_no%1000) && !quiet)
    printf ("%d lines\n", line_no);
  stripchar (line, '\n');
  if (record_separator(line)) {
    offset = ftell(fp);
    do_increment_rec_count = TRUE;
    }
  else if (get_tuple(object_name, attribute_name, datum, line) && 
    object_name[0] != '_') {
    if ((ifp = get_object_db (object_name, data_file, mode, type, 
      &file_no))!=NULL) {
      if (do_increment_rec_count) {
         increment_record_count (object_name, data_file);
         do_increment_rec_count = FALSE;
         }
      pass=0;
      while (set_index_format (object_name, attribute_name, line, datum, 
        pass++)) 
{
/* fprintf (stdout, "%d:%d:%d:%s\n", offset, file_no, FALSE, line); */
        fprintf (ifp, "%d:%d:%d:%s\n", offset, file_no, FALSE, line);
}
      }
    }
  }
fclose (fp);
}

/*************************************************************************
 shows the status of the indexer
**************************************************************************/
show_index_startup_status(mode, type, default_dir, config_file)
int mode;
int type;
char *default_dir;
char *config_file;

{
if (mode == INITIALIZE)
  printf ("mode: initialize\n");
else if (mode == APPEND)
  printf ("mode: append\n");
if (type == AUTHORITATIVE)
  printf ("type: authoritative\n");
else
  printf ("type: cached\n");
if (default_dir != NULL)
  printf ("config_file: %s/%s\n", default_dir, config_file);
else
  printf ("config_file: %s\n", config_file);
}
