/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"
#include "fileinfo.h"
#include "conf.h"
#include "objects.h"
#include "client_msgs.h"
#include "rwhois_supp.h"
#include "llist.h"
#include "io.h"

static int mode;

/*****
  local macros
******/

#define DB_BASE_TEMPLATE "%s.db"
#define DB_UNSORTED_TEMPLATE "%s-%d.sort"
#define DB_INDEX_TEMPLATE "%s-%d.ndx"
#define DATA_FILE             "DATA"
#define INDEX_FILE            "INDEX"


/******
  local structure
*******/

typedef struct _rwhois_io_struct {
  char object_name[MAX_LINE];
  int type;
  FILE *base_fp;
  FILE *index_fp;
  char index_unsorted[MAX_FILENAME];
  char index_sorted[MAX_FILENAME];
  int file_no;
  int num_recs;
  int total_recs;
  int mode;
  char data_file[MAX_FILENAME];
} rwhois_io_struct;

#define NEW_RWHOIS_IO (rwhois_io_struct *) calloc ((unsigned)1,\
  (unsigned)sizeof(rwhois_io_struct))  

/******
  local protos
*******/

/*******
  local globals
********/

static char data_dir[MAX_FILENAME];
static char rwhois_root[MAX_FILENAME];
llist_struct *open_object_db_list = NULL;

/************************************************************************
sets the mode of all the required files
*************************************************************************/
set_db_file_mode (passed_mode)
int passed_mode;

{
mode = passed_mode;
}

/************************************************************************
saves the current working directory
*************************************************************************/
save_data_dir ()
{
getcwd (data_dir, MAX_FILENAME); /* thanks to A.Linton@aarnet.edu.au */
}

/************************************************************************
saves the current working directory
*************************************************************************/
save_rwhois_root ()
{
getcwd (rwhois_root, MAX_FILENAME); /* thanks to A.Linton@aarnet.edu.au */
}

/************************************************************************
saves the current working directory
*************************************************************************/
chdir_data_dir ()
{
chdir (data_dir);
}

/************************************************************************
saves the current working directory
*************************************************************************/
chdir_rwhois_root ()
{
chdir (rwhois_root);
}

/*************************************************************************
 changes file location relative to rwhois root, tags as null if
 int in root dir
**************************************************************************/
char *chg_file_relative_to_rwhois_root (file)
char *file;

{
char *d, *r;
char *rel_to_rwhois_root;

if (strlen(file) >= strlen(rwhois_root))
  {
  if (strncmp (file, rwhois_root, strlen(rwhois_root)) != 0)
    /* wrong base */
    return (NULL);
  else
    d = file+strlen(rwhois_root)+1;
    return (d);
  }
return (file);
}

/*************************************************************************
 gives location of data file relative to the root
**************************************************************************/
char *relative_to_rwhois_root ()

{
static char rel_to_rwhois_root[MAX_FILENAME];
char *d, *r;

*rel_to_rwhois_root='\0';
if (strlen(data_dir) >= strlen(rwhois_root))
  {
  if (strncmp (data_dir, rwhois_root, strlen(rwhois_root)) != 0)
    /* wrong base */
    strcpy (rel_to_rwhois_root, data_dir);
  else if (strlen(data_dir) == strlen(rwhois_root))  
    /* same path */
     ;
  else {
    /* longer path - just append on */
    d = data_dir;
    d+= strlen(rwhois_root);
    if (*d != '/') {
      strcpy (rel_to_rwhois_root, data_dir);
      return (data_dir);
      }
    d++;
    r = rel_to_rwhois_root;
    while (*d != '\0') {
      *r = *d;
      r++; d++;
      }
    *r = '\0';
    }
  }
else 
  /* shorter - needs full path */
  strcpy (rel_to_rwhois_root, data_dir);
return (rel_to_rwhois_root);
}


/*************************************************************************
 opens the files
**************************************************************************/
FILE *open_file (db_file, mode)
char *db_file;
char *mode;

{
FILE *fp;

if((fp = fopen(db_file, mode)) == (FILE *) NULL){
  print_error (MISC_DIAG, "can not open %s mode %s", db_file, mode);
  return(NULL);
  }
return (fp);
}

/*************************************************************************
 opens the index files
**************************************************************************/
FILE *get_object_db (object_name, data_file, mode, type, file_no)
char *object_name;
char *data_file;
int mode;
int type;
int *file_no;

{
object_struct *object;
rwhois_io_struct *ptr;
llist_struct *cur = open_object_db_list;
char *base_name;

/****
  sees if the object info is already open
  if it is and same filename - cool
    else closes out the filename and starts a new one with a new file_no
****/

while (cur != NULL) {
  ptr = cur->datum;
  if (strcasecmp (ptr->object_name, object_name) == 0) {
    /* got a new file - save info on old one */
    if (strcmp (data_file, ptr->data_file) !=0) {
      save_in_db (ptr->base_fp, ptr->data_file, DATA_FILE, ptr->file_no,
        ptr->num_recs);
      strcpy (ptr->data_file, data_file);
      ptr->num_recs=0;
      ptr->file_no++;
      }
    *file_no = ptr->file_no;
    return (ptr->index_fp);
    }
  cur = cur->nxt;
  }

/*****
  changes to the rwhois root for opening up the new object stuff
******/

if ((object = get_object_by_object_name (object_name)) == NULL) {
  print_error (MISC_DIAG, "open_object_db: can not find object '%s'", 
    object_name);
  return (NULL);
  }

chdir_rwhois_root();
ptr = NEW_RWHOIS_IO;
strcpy (ptr->object_name, object_name);
strcpy (ptr->data_file, data_file);
ptr->num_recs = 0;
ptr->total_recs = 0;
if (type == AUTHORITATIVE) 
  base_name = object->local_db;
else
  base_name = object->cache_db;
if (mode == INITIALIZE) 
  initialize_rwhois_db (base_name, ptr);
else 
  append_rwhois_db (base_name, ptr);
*file_no = ptr->file_no;

/*****
  changes back to the data dir and appends the object stuff to
  a list
*****/

chdir_data_dir();
if (ptr->base_fp != NULL)
  open_object_db_list = append_llist (open_object_db_list, ptr);
return (ptr->index_fp);
}

/*************************************************************************
 initializes the database
**************************************************************************/
initialize_rwhois_db (base_name, ptr)
char *base_name;
rwhois_io_struct *ptr;

{
char basefname[MAX_FILENAME];

sprintf (basefname, DB_BASE_TEMPLATE, base_name);
clear_existing_indexes (basefname);
ptr->base_fp = open_file (basefname, "w");
sprintf (ptr->index_unsorted, DB_UNSORTED_TEMPLATE, base_name, 0);
sprintf (ptr->index_sorted, DB_INDEX_TEMPLATE, base_name, 0);
ptr->index_fp = open_file (ptr->index_unsorted, "w");
}

/*************************************************************************
 appends info to the database
**************************************************************************/
append_rwhois_db (base_name, ptr)
char *base_name;
rwhois_io_struct *ptr;

{
char basefname[MAX_FILENAME];
mkdb_fileinfo_struct *fileinfo = NULL;
int num_indexes;

sprintf (basefname, DB_BASE_TEMPLATE, base_name);
if ((ptr->base_fp = open_file(basefname, "r+")) != NULL) 
  fileinfo = load_fileinfo_list (ptr->base_fp);
ptr->file_no = next_file_no_from_fileinfo (fileinfo);
num_indexes = next_index_no_from_fileinfo (fileinfo);
sprintf (ptr->index_unsorted, DB_UNSORTED_TEMPLATE, base_name, num_indexes);
sprintf (ptr->index_sorted, DB_INDEX_TEMPLATE, base_name, num_indexes);
ptr->index_fp = open_file (ptr->index_unsorted, "w");
clear_fileinfo_list (fileinfo);
}


/*************************************************************************
 sets the next fileinfo number from the list
 returns the next available number
**************************************************************************/
next_file_no_from_fileinfo (fileinfo)
mkdb_fileinfo_struct *fileinfo;

{
int max = 0;

while (fileinfo != NULL) {
  if (fileinfo->index_no > max)
    max = fileinfo->index_no;
  fileinfo=fileinfo->nxt;
  }
return (max+1);
}

/*************************************************************************
 returns the next available index number
**************************************************************************/
next_index_no_from_fileinfo (fileinfo)
mkdb_fileinfo_struct *fileinfo;

{
int cnt = 0;

while (fileinfo != NULL) {
  if (fileinfo->type == ENUM_INDEX_FILE)
     cnt++;
  fileinfo=fileinfo->nxt;
  }
return (cnt);
}

/*************************************************************************
 removes any existing indexes
**************************************************************************/
clear_existing_indexes (basefname)
char *basefname;

{
FILE *fp;
mkdb_fileinfo_struct *fileinfo, *cur;

if ((fp = open_file(basefname, "r")) != NULL) {
  cur = fileinfo = load_fileinfo_list (fp);
  while (cur != NULL) {
    if (cur->type == ENUM_INDEX_FILE)
      unlink (cur->filename);
    cur = cur->nxt;
    }
  clear_fileinfo_list (fileinfo);
  }
}

/*************************************************************************
 saves file info into db
**************************************************************************/
int save_in_db (fp, data_file, type, file_no, num_recs)
FILE *fp;
char *data_file;
char *type;
int file_no;
int num_recs;

{
struct stat file_info;

stat(data_file, &file_info);
if (file_no > 0)
  fprintf (fp, "---\n");
fprintf (fp, "%s: %s\n", MKDB_TYPE, type);
fprintf (fp, "%s: %s\n", MKDB_FILE, data_file);
fprintf (fp, "%s: %d\n", MKDB_INDEX_NO, file_no);
fprintf (fp, "%s: %d\n", MKDB_NUMRECS, num_recs);
fprintf (fp, "%s: %d\n", MKDB_SIZE, file_info.st_size);
}

/*************************************************************************
 increments the record count based on name and data file
**************************************************************************/
increment_record_count (object_name, data_file)
char *object_name, *data_file;

{
rwhois_io_struct *ptr;
llist_struct *cur = open_object_db_list;

while (cur != NULL) {
  ptr = cur->datum;
  if (strcasecmp (ptr->object_name, object_name) == 0 && 
    strcmp (data_file, ptr->data_file) ==0)  {
    ptr->num_recs++;
    ptr->total_recs++;
    return;
    }
  cur = cur->nxt;
  }
}

/*************************************************************************
 closes the existing data files, sorts, and creates a index record
**************************************************************************/
flush_and_sort (quiet, in_jail)
int quiet;
int in_jail;

{
rwhois_io_struct *ptr;
llist_struct *cur = open_object_db_list;

while (cur != NULL) {
  ptr = cur->datum;
  /* save in the data file stuff */
  save_in_db (ptr->base_fp, ptr->data_file, DATA_FILE, ptr->file_no,
    ptr->num_recs); 

  /* change to the rwhois root area, close the index file, and sort it */
  chdir_rwhois_root();
  fclose (ptr->index_fp);
  sort_index (quiet, in_jail, ptr->index_unsorted, ptr->index_sorted);
  save_in_db (ptr->base_fp, ptr->index_sorted, INDEX_FILE, ptr->file_no+1,
    ptr->total_recs);
  chdir_data_dir();

  /* close the db file since on to the next object */
  fclose (ptr->base_fp);
  cur = cur->nxt;
  }
}

/*************************************************************************
 sorts the index file
**************************************************************************/
sort_index (quiet, in_jail, unsorted_name, sorted_name)
int quiet;
int in_jail;
char *unsorted_name;
char *sorted_name;

{
char buf[MAX_BUF];
char prog[MAX_BUF];
char params[MAX_BUF];

if (!quiet)
  printf ("sorting %s ...", sorted_name);

if (in_jail)
  sprintf (buf,"/bin/sort -i +3 -t':' %s -o %s", unsorted_name, sorted_name);
else
  sprintf (buf,"/usr/bin/sort -i +3 -t':' %s -o %s", unsorted_name, 
    sorted_name);
if (system (buf)!=0) { 

/*
if (in_jail)
  strcpy (prog, "/bin/sort");
else
  strcpy (prog, "/usr/bin/sort");
sprintf (params, "-i +3 -t':' %s -o %s", unsorted_name, sorted_name);
if (!run_program (prog, params)!=0) {
*/

  print_error (MISC_DIAG, "sort failed!");
  exit(1);
  }
unlink (unsorted_name);
if (!quiet)
  printf ("done!\n");
}

/*************************************************************************
 makes sure the data directory is in the rwhois root area
**************************************************************************/
data_dir_in_bounds()
{
printf ("data_dir_in_bounds: looking at |%s|\n", rwhois_root);
printf ("data_dir_in_bounds: looking at |%s|\n", data_dir);
if (strncmp (data_dir, rwhois_root, strlen(rwhois_root)) == 0) {
printf ("data_dir_in_bounds: ok on strncmp\n");
printf ("data_dir_in_bounds: looking at |%c|\n", *(data_dir+strlen(rwhois_root))); 
  if (*(data_dir+strlen(rwhois_root)) == '/') {
printf ("data_dir_in_bounds: ok\n"); 
    return (TRUE);
}
}
return (FALSE);
}
