/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
  Stan Borinski
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"
#include "fileinfo.h"
#include "client_msgs.h"
#include "conf.h"
#include "llist.h"
#include "io.h"
#include "validate.h"
#include "parse.h"

#define USAGE_STR "%s:  <-j> <-c rwhois_config_file> [files]\n"

#define OUTPUT_FILE_FORMAT   "%s.txt"

#include  <string.h>

#ifndef TRUE
#define FALSE 0
#define TRUE  1
#endif
#define ERROR -1

/* Find the max string length macro used by the rest of the program later */
#define MAX_STRING 80

/**********
  local globals
***********/


/************
  local prototypes
*************/
static int get_header_info();

/*************************************************************************
  main routine
**************************************************************************/
main(argc, argv)
int argc;
char *argv[];

{
extern int optind;
extern char *optarg;
char c;
int no_err = TRUE;
char relative_path_name[MAX_FILENAME];
char email[MAX_LINE], action[MAX_LINE];
llist_struct *record_list;
char *config_file = NULL;
int in_jail = FALSE;

/******
  initialization
******/

save_data_dir();

/******
  command line parsing
*******/

setbuf (stdout, NULL);
while ((c = getopt(argc, argv, "jc:")) != EOF) {
  switch (c) {
    case 'c':
      config_file = optarg;
      break;
    case 'j':
      in_jail = TRUE;
      break;
    }
  }
if (argc == 1) {
  fprintf (stderr, USAGE_STR, argv[0]);
  exit (1);
  }

/********
  read in rwhois configuration items
********/

if (config_file == NULL)
  config_file = DEFAULT_RWHOIS_CONFIG_FILE;

if (!read_config_files(config_file, in_jail))   {
  printf ("problem with config files\n");
  exit (1);
  }
save_rwhois_root();

/*******
  change back to current directory and index
*******/

while (optind<argc && no_err) {
  if (strlen (relative_to_rwhois_root()) > 0 && *argv[optind] != '/')
    sprintf (relative_path_name, "%s/%s", relative_to_rwhois_root(),
      argv[optind]);
  else
    strcpy (relative_path_name, argv[optind]);
  if ((no_err = parse_file (relative_path_name, &record_list, action, email))) 
    no_err = update_database (action, record_list, relative_path_name); 
  optind++;
  } 
}

/*************************************************************************
 parses the file
 RETURNS  TRUE if ok
          FALSE if not
**************************************************************************/
int  parse_file (data_file, top, action, email)
char *data_file;
llist_struct **top;
char *action;
char *email;

{
char output_file[MAX_LINE], line[MAX_LINE];
char version[MAX_LINE];
llist_struct *record_list = NULL, *record = NULL;
line_struct *new;
FILE *ifp, *ofp;
int row_no=0;
int retval = TRUE;

/******
  first read in record
******/

if ((ifp = open_file (data_file, "r")) == NULL) {
  fprintf (stderr, "parse_file: can not open %s\n", data_file);
  return (FALSE);
  }
sprintf (output_file, OUTPUT_FILE_FORMAT, data_file);
if ((ofp = open_file (output_file, "w")) == NULL) {
  fprintf (stderr, "parse_file: can not open %s\n", data_file);
  return (FALSE);
  }

if (!set_header_info (ifp, ofp, version, email, action, &row_no)) {
  fprintf (stderr, "parse_file: can not do header!\n");
  fclose (ifp);
  fclose (ofp);
  return (FALSE);
  }

while (fgets (line, MAX_LINE, ifp) != NULL) {
  row_no++;
  fputs (line, ofp);
  stripchar (line, '\n');
  trim (line);
  if (strlen (line) == 0)  {
    if (record != NULL)
      record_list = append_llist (record_list, record);
    record = NULL;
    }
  else {
    new = NEW_LINE_STRUCT;
    new->row_no = row_no;
    new->line = NEW_STRING (line);
    record = append_llist (record, new);
    }
  }
fclose (ifp);

/***
  if no end of record sent before EOF
****/

if (record != NULL) 
  record_list = append_llist (record_list, record);  

/*******
  now validate record
*******/
  
*top = record_list;
while (record_list != NULL) {
  if (!validate_record (ifp, ofp, record_list->datum)) {
/*
     fprintf (stderr, "parse_file: bombed on validate_record\n");
*/
     retval = FALSE;
     }
  record_list = record_list->nxt;
  }
fclose (ofp);

/*******
  print out the errors if found
********/

/*
if (!retval) {
  if ((ofp = open_file (output_file, "r")) == NULL) {
    fprintf (stderr, "parse_file: can not open %s\n", output_file);
    return (FALSE);
    }
  while (fgets (line, MAX_LINE, ofp) != NULL) {
    stripchar (line, '\n');
    printf ("%s\n", line);
    }
  fclose (ofp);
  }
*/
unlink (output_file);

/*
debug_record_list (record_list);
debug_record_list (record_list);
*/
return (retval);
}

/*************************************************************************
sets the version #, email address, and action desired
**************************************************************************/
static int set_header_info (ifp, ofp, version, email, action, row_no)
FILE *ifp, *ofp;
char *version, *email, *action;
int *row_no;

{
char line[MAX_LINE], tag[MAX_LINE], datum[MAX_LINE];
int i;

*version='\0';
*email='\0';
*action='\0';
for (i=0; i<3; i++) {
  (*row_no)++;
  if (fgets (line, MAX_LINE, ifp) != NULL)  {
    fputs (line, ofp);
    stripchar (line, '\n');
    if (parse_line (line, tag, datum)) {
      if (strcasecmp (tag, H_VERSION) == 0)
        strcpy (version, datum);
      else if (strcasecmp (tag, H_EMAIL) == 0)
        strcpy (email, datum);
      else if (strcasecmp (tag, H_ACTION) == 0)
        strcpy (action, datum);
      }
    }
  else {
    print_error_to_file (ifp, ofp, TRUE, *row_no, 
      "Undefined Header tag '%s'\n", tag);
    return (FALSE);
    }
  } 
if (strlen (version)==0 || strlen(email)==0 || strlen(action)==0) {
  print_error_to_file (ifp, ofp, TRUE, *row_no, "Required Header missing");
  return (FALSE);
  }
return (TRUE);
}

debug_record_list (rec_list)
llist_struct *rec_list;

{
llist_struct *rec;
line_struct *line;

printf ("debug_record: on!\n");
while (rec_list != NULL) {
  debug_record (rec_list->datum);
  rec_list=rec_list->nxt;
  }
printf ("debug_record: done!\n");
}

debug_record (att)
llist_struct *att;

{
line_struct *line;
while (att!=NULL) {
 line = att->datum;
 printf ("%d: %s\n", line->row_no, line->line);
 att=att->nxt;
 }
}
